package audio;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Random;

public class CreateWav1 {

    static class Sample {

        public Short leftSample;
        public Short rightSample;

        public Sample(Short left, Short right) {
            leftSample = left;
            rightSample = right;
        }

    }

    static class WaveData {

        private ArrayList<Sample> audioSamples = new ArrayList<>();

        public boolean saveSamplesToDisk(String pathFileName) {

            File file = new File(pathFileName);
            if (!file.exists()) {
                try {
                    file.createNewFile();
                } catch (IOException ex) {
                    return false;
                }
            }

            byte[] bytes = new byte[audioSamples.size() * 4];
            for (int i = 0; i < audioSamples.size(); i++) {
                bytes[4 * i] = (byte) (audioSamples.get(i).leftSample & 0xff);
                bytes[4 * i + 1] = (byte) ((audioSamples.get(i).leftSample >> 8) & 0xff);
                bytes[4 * i + 2] = (byte) (audioSamples.get(i).rightSample & 0xff);
                bytes[4 * i + 3] = (byte) ((audioSamples.get(i).rightSample >> 8) & 0xff);
            }

            // scrittura su disco tramite FileWriter e BufferedWriter
            try {
                FileOutputStream fos = new FileOutputStream(file);
                fos.write(WavHeader.returnHeader(audioSamples.size(), 44100, 2, 16));
                fos.write(bytes);
                fos.close();
            } catch (IOException ex) {
                return false;
            }
            return true;
        }

        public void generateWhiteNoise(int nSamples, double normalizedAmplitude) {
            audioSamples.clear();

            if (Math.abs(normalizedAmplitude) > 1) {
                normalizedAmplitude = 1;
            }

            Random rand = new Random();
            for (int i = 0; i < nSamples; i++) {
                short leftCandidate = (short) ((rand.nextInt(65535) - 32768) * Math.abs(normalizedAmplitude));
                short rightCandidate = (short) ((rand.nextInt(65535) - 32768) * Math.abs(normalizedAmplitude));
                Sample s = new Sample(leftCandidate, rightCandidate);
                audioSamples.add(s);
            }
        }

        public void generateSawTooth(int nSamples, double normalizedAmplitude, double frequency) {
            audioSamples.clear();

            if (Math.abs(normalizedAmplitude) > 1) {
                normalizedAmplitude = 1;
            }
            short amplitude = (short) (32767 * Math.abs(normalizedAmplitude));

            double nSamplesPerCycle = 44100 / frequency;
            for (int k = 0; k < nSamples; k++) {
                int currentSample = (int) (k % nSamplesPerCycle);
                short candidate = (short) ((currentSample * (amplitude * 2) / nSamplesPerCycle) - amplitude);
                Sample s = new Sample(candidate, candidate);
                audioSamples.add(s);
            }
        }

        public void generateSine(int nSamples, double normalizedAmplitude, double frequency) {
            audioSamples.clear();

            if (Math.abs(normalizedAmplitude) > 1) {
                normalizedAmplitude = 1;
            }
            short amplitude = (short) (32767 * Math.abs(normalizedAmplitude));

            double nSamplesPerCycle = 44100 / frequency;
            for (int k = 0; k < nSamples; k++) {
                short candidate = (short) (Math.sin(2 * Math.PI * k / nSamplesPerCycle) * amplitude); // (k / nSamplesPerCycle) calcola il punto corrente sulla circonferenza goniometrica per il calcolo del seno
                Sample s = new Sample(candidate, candidate);
                audioSamples.add(s);
            }
        }
    }

    /**
     * @param args the command line arguments
     */
    public static void main(String[] args) {
        WaveData wd = new WaveData();
        //wd.generateWhiteNoise(44100, 0.5);
        //wd.generateSawTooth(44100, 1, 440);
        wd.generateSine(44100, 0.8, 440);
        wd.saveSamplesToDisk("prova.wav");
    }

}
