package audio;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.IOException;

public class CustomWavetable {

    public static void main(String[] args) throws FileNotFoundException, IOException {

        final int sampleRate = 22050;

        double[] myWavetable = new double[32];
        int noteFrequency = 220;
        int nSamples = 22050;
        double amplitude = 0.7;

        // caricamento della wavetable
        BufferedReader br = null;
        try {
            String currentLine;
            br = new BufferedReader(new FileReader("wavetable.txt"));

            int i = 0;
            while ((currentLine = br.readLine()) != null)
                myWavetable[i++] = Double.parseDouble(currentLine);

        } catch (IOException | NumberFormatException e) {
        } finally {
            try {
                if (br != null)
                    br.close();
            } catch (IOException e) {
            }
        }

        int nSamplesPerCycle = (int) (sampleRate / noteFrequency);
        byte[] result = new byte[nSamples];

        // lettura ciclica della tabella
        for (int k = 0; k < nSamples; k++) {
            int index = k % nSamplesPerCycle;
            double rescaledIndex = ((double) index * myWavetable.length / nSamplesPerCycle);
            int floor = (int) Math.floor(rescaledIndex);
            double temp = 0;
            if (floor + 1 < myWavetable.length)
                temp = (rescaledIndex - floor) * (myWavetable[floor + 1] - myWavetable[floor]) + myWavetable[floor];
            else
                temp = myWavetable[floor];
            result[k] = (byte) (temp * amplitude * 128);
        }

        // salvataggio su disco
        File file = new File("result.raw");
        if (!file.exists())
            try {
                file.createNewFile();
            } catch (IOException ex) {
                System.exit(1);
            }
        try (FileOutputStream fos = new FileOutputStream(file)) {
            fos.write(result);
        }
    }

}
