package audio;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;

public class Modulations1 {

    static class Modulations {

        private static final int sampleRate = 22050;

        public static byte[] getRM(int nSamples, double fc, double Ac, double fm, double Am) {
            byte[] result = new byte[nSamples];
            double[] resultDouble = new double[nSamples];
            double nSamplesPerCycleC = sampleRate / fc;
            double nSamplesPerCycleM = sampleRate / fm;
            double max = 0;
            for (int i = 0; i < nSamples; i++) {
                double modulator = Am * Math.sin(2 * Math.PI * (i / nSamplesPerCycleM));
                resultDouble[i] = modulator * (Ac * Math.sin(2 * Math.PI * (i / nSamplesPerCycleC)));
                if (max < Math.abs(resultDouble[i]))
                    max = Math.abs(resultDouble[i]);
            }
            for (int i = 0; i < nSamples; i++)
                result[i] = (byte) (resultDouble[i] / max * 0.9 * 127);
            return result;
        }

        public static byte[] getAM(int nSamples, double fc, double Ac, double fm, double Am) {
            byte[] result = new byte[nSamples];
            double[] resultDouble = new double[nSamples];
            double nSamplesPerCycleC = sampleRate / fc;
            double nSamplesPerCycleM = sampleRate / fm;
            double max = 0;
            for (int i = 0; i < nSamples; i++) {
                double modulator = Am * Math.sin(2 * Math.PI * (i / nSamplesPerCycleM)) + Am;
                resultDouble[i] = (Ac + modulator) * Math.sin(2 * Math.PI * (i / nSamplesPerCycleC));
                if (max < Math.abs(resultDouble[i]))
                    max = Math.abs(resultDouble[i]);
            }
            for (int i = 0; i < nSamples; i++)
                result[i] = (byte) (resultDouble[i] / max * 0.9 * 127);
            return result;
        }
    }

    public static void main(String[] args) throws FileNotFoundException, IOException {

        byte[] AMResult = Modulations.getAM(44100, 800, 1, 100, 0.3);
        byte[] RMResult = Modulations.getRM(44100, 600, 1, 50, 0.5);
        byte[] tremolo = Modulations.getAM(44100, 800, 1, 8, 0.3);    // frequenza della modulante bassa

        // salvataggio su disco
        File file = new File("amresult.raw");
        if (!file.exists())
            try {
                file.createNewFile();
            } catch (IOException ex) {
                System.exit(1);
            }
        try (FileOutputStream fos = new FileOutputStream(file)) {
            fos.write(AMResult);
        }

        file = new File("rmresult.raw");
        if (!file.exists())
            try {
                file.createNewFile();
            } catch (IOException ex) {
                System.exit(1);
            }
        try (FileOutputStream fos = new FileOutputStream(file)) {
            fos.write(RMResult);
        }

        file = new File("tremolo.raw");
        if (!file.exists())
            try {
                file.createNewFile();
            } catch (IOException ex) {
                System.exit(1);
            }
        try (FileOutputStream fos = new FileOutputStream(file)) {
            fos.write(tremolo);
        }
    }
}
