package jMusic;

import jm.music.data.*;
import jm.JMC;
import jm.util.*;

public class RhythmicAutomata2 implements JMC {

    final int CA = 30;
    /* declare the empty bar arrays */
    int barLength = 8;
    int[] bar = new int[barLength];
    int[] tempBar = new int[barLength];
    /* Declare the jMusic score and parts */
    Part part1 = new Part("Hats", 0, 9);
    Part part2 = new Part("Rim Shot", 0, 9);
    Part part3 = new Part("Claps", 0, 9);
    Score score = new Score("AlgoRhythm2", 84);

    /**
     * The program starts here first
     */
    public static void main(String[] args) {
        new RhythmicAutomata2();
    }

    /**
     * Constructor
     */
    public RhythmicAutomata2() {
        /* fill array */
        initialise();
        makeMusic(part1, 42);

        /* calc next array */
        for (int i = 0; i < 23; i++) {
            nextBar();
            makeMusic(part1, 42);
            printBar();
        }

        /* fill array */
        initialise();
        makeMusic(part2, 37);

        /* calc next array */
        for (int i = 0; i < 23; i++) {
            nextBar();
            makeMusic(part2, 37);
            printBar();
        }

        /* fill array */
        initialise();
        makeMusic(part3, 39);

        /* calc next array */
        for (int i = 0; i < 23; i++) {
            nextBar();
            makeMusic(part3, 39);
            printBar();
        }

        score.addPart(part1);
        score.addPart(part2);
        score.addPart(part3);

        Play.midi(score);
    }

    /**
     * Fill the bar with randomly set values
     */
    private void initialise() {
        System.out.println();
        System.out.println("====== Initialising automata array ======");
        for (int i = 0; i < barLength; i++)
            bar[i] = (int) (Math.random() * 2);
        printBar();
    }

    /**
     * Display the bar values on the command line
     */
    private void printBar() {
        for (int i = 0; i < barLength; i++)
            System.out.print(bar[i] + " ");
        System.out.println();
    }

    /**
     * Apply the rules to generate the next bar
     */
    private void nextBar() {
        /* calculate the number of neighbours */
        for (int i = 0; i < barLength; i++) {
            int[] cells = new int[3]; // represents the cell and its neighbours
            
            int index = 0;
            for (int j = i-1; j <= i+1; j++) {
                if (j < 0) // in case of first cell the previous neighbour is 0
                    cells[index] = 0;
                else if (j > barLength - 1) // in case of last cell the next neighbour is 0
                    cells[index] = 0;
                else
                    cells[index] = bar[j];
                index++;
            }
            tempBar[i] = applyRule(cells);
        }
        /* shift new values in */
        bar = tempBar;
    }
    
    private int applyRule(int[] cells) {
        int p = cells[0] * 4 + cells[1] * 2 + cells[2];
        int newCell = (CA & (int)Math.pow(2, p)) > 0 ? 1 : 0;
        return newCell;
    }

    /**
     * Convert the bar array into a jMusic phrase and add the phrase to the
     * specified part.
     */
    private void makeMusic(Part p, int pitch) {
        Phrase phr = new Phrase();
        for (int i = 0; i < barLength; i++)
            if (bar[i] == 0)
                phr.addNote(new Note(REST, SQ));
            else
                phr.addNote(new Note(pitch, SQ, (int) (Math.random() * 60 + 60)));
        p.addPhrase(phr);
    }
}
